//
// QPerforce is a gui interface to the perforce revision control
// system. It is based on the qt library, and should be easily portable
// across any platform that has this toolkit available.
// 
// Copyright (C) 2002 Jacob Gladish
// 
// TODO: find legit copyright.
//
#ifndef __CHANGE_LIST_MENU_H__
#define __CHANGE_LIST_MENU_H__

#include "IUpdateable.h"
#include "Change.h"
#include "WidgetId.h"

#include <qlistview.h>
#include <qpopupmenu.h>

class QRadioButton;
class QButtonGroup;
class PendingChangeListsTree;
class SubmittedChangeListsTable;

class ChangeListMenu : public QPopupMenu
{
  Q_OBJECT

public:

	ChangeListMenu( QWidget* = 0, const char* name = 0 );
	virtual ~ChangeListMenu();

  void setSubmittedChangeListsView( SubmittedChangeListsTable* table );
  void setPendingChangeListsView( PendingChangeListsTree* tree );

  typedef void (ChangeListMenu::*ChangeListMenuFuncPtr) ();

signals:

  void showSubmittedChangeLists( WidgetId id );
  void showPendinChangeLists( WidgetId id );

public slots:

	void slotViewSubmittedChangelists();
	void slotViewPendingChangelists();
	
  // void slotFilterChangelists( int );
  void slotActivated( int );

	void slotSortChangesByUser();
	void slotSortFilesInPendingChangelistsByExtension();
	void slotIntegrate();
	void slotSyncToChangeList();
	void slotRevertUnchangedFiles();
	void slotRemoveJobFix();
	void slotDescribeJobFix();
	void slotAddJobFix();
	void slotSubmit();
	void slotDelete();
	void slotDescribeChangeList();
	void slotDescribe();
	void slotEditSpec();
	void slotNew();

private:

  void _viewSubmittedChangelists();
  void _viewPendingChangelists();
  
  int _unfilteredSubmittedChangelistsId;
  int _fscForSelectedFilesId;
  int _fscForSelectedFilesIncludingIntegrationsId;
  int _fscByClientId;
  int _fscByUserId;

  int _ids[ 5 ];

  SubmittedChangeListsTable* _submittedChangeListsTable; 
  PendingChangeListsTree*    _pendingChangeListsTree;
};



//========================================================================
// The Widget to display submitted changes
//
class SubmittedChangeListsTable : public QListView 
{

public:

  SubmittedChangeListsTable( QWidget* parent = 0, const char* name = 0 ); 
  void addSubmittedChangeList( const Perforce::Change* scl );

};


//========================================================================
// The Widget to display pending changes
//
class PendingChangeListsTree : public QListView 
{

  Q_OBJECT

public:

  PendingChangeListsTree( QWidget* parent = 0, const char* name = 0 );
  void addChange( const Perforce::Change* pcl );
  void clear();  

public slots:

  void slotExpanded( QListViewItem* item );

private:

  QListViewItem* _currentClient;  
  QListViewItem* _everyoneElse;
};


#endif


