//
// QPerforce is a gui interface to the perforce revision control
// system. It is based on the qt library, and should be easily portable
// across any platform that has this toolkit available.
// 
// Copyright (C) 2002 Jacob Gladish
//
#ifndef __FILE_MENU_H__
#define __FILE_MENU_H__

#include "clientapi.h"
#include <qpopupmenu.h>

class FileMenu  : public QPopupMenu
{

  Q_OBJECT

public:

  FileMenu( QWidget* parent = 0, const char* name = 0 );
  virtual ~FileMenu();

private:

  QPopupMenu* _sync;
  QPopupMenu* _integrate;
  QPopupMenu* _resolve;

  struct Runner : public ClientUser {
    void spawnConsole();
  } _runner;

public slots:
  
  void slotAddToSourceControl();

  //----------------------------------------
  // sync
  void slotSyncToHeadRevision();
  void slotSyncToHeadRevisionPreviewOnly();
  void slotSyncTORevisionLabelChangeListNumberOrDate();
  void slotRedoSync();

  //----------------------------------------

  void slotOpenForEdit();
  void slotChangeType();

  //----------------------------------------

  void slotOpenForDelete();
  void slotRevert();
  void slotRecoverDeletedFile();
  void slotRemoveFromClient();

  //----------------------------------------

  void slotUnlock();
  void slotLock();

  //----------------------------------------

  void slotAssociatedEditor();
  void slotViewFile();

  //----------------------------------------

  void slotRevisionHistory();
  void slotProperties();
  void slotCheckConsistency();
  void slotAddFilesToUserReviews();
  void slotAddFilesToClientView();
  void slotDiffClientFileVSDepotFile();

  //----------------------------------------

  // -- Integrate --
  void slotIntegrateUsingBranchspec();
  void slotIntegrateUsingFilespec();  

  // -- Resolve ---
  void slotScheduleFileForResolve();
  void slotResolveAutoResolve();
  void slotResolveInteractively();

  //----------------------------------------

  void slotExplore();
  void slotPageSetup();
  void slotCommandPrompt();
  void slotExit();

};

#endif 
